@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 #
-# $Id: autogen.sh,v 1.14 2003/12/24 13:04:09 f1rmb Exp $
+# $Id: autogen.sh,v 1.15 2004/07/18 20:56:33 f1rmb Exp $
 #
 # Maintained by Stephen Torri <storri@users.sourceforge.net>
 #
@@ -224,7 +224,7 @@ run_aclocal () {
   fi
   
   echo $_echo_n " + Running aclocal: $_echo_c"
-  aclocalinclude=`xine-config --acflags`
+  aclocalinclude=`if [ ! -z "$XINE_CONFIG" ]; then $XINE_CONFIG --acflags; else xine-config --acflags; fi`
 
   aclocal -I m4 $aclocalinclude
   echo "done." 
