@@ -77,8 +77,16 @@ struct fbxine
 	int                      debug;
 
 	int                      ignore_next;
-        post_element_t           **post_elements;
-        int                      post_elements_num;
+
+        post_element_t          **post_elements;
+        int                       post_elements_num;
+        int                       post_enable;
+        
+        char                     *deinterlace_plugin;
+        post_element_t          **deinterlace_elements;
+        int                       deinterlace_elements_num;
+        int                       deinterlace_enable;
+  
   
 #ifdef HAVE_LIRC
 	struct
@@ -100,6 +108,9 @@ struct fbxine
     int                     enabled;
     int                     timeout;
 
+    xine_osd_t             *sinfo;
+    int                     sinfo_visible;
+
     xine_osd_t             *bar[2];
     int                     bar_visible;
 
@@ -116,3 +127,10 @@ struct fbxine
 
 extern struct fbxine fbxine;
 void fbxine_exit(void);
+
+#define SAFE_FREE(x)            do {           \
+                                  if((x)) {    \
+                                    free((x)); \
+                                    x = NULL;  \
+                                  }            \
+                                } while(0)
