@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: file_browser.c,v 1.49 2004/04/13 19:55:47 f1rmb Exp $
+ * $Id: file_browser.c,v 1.58 2004/12/12 14:19:08 valtri Exp $
  *
  */
 
@@ -29,7 +29,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#include <X11/cursorfont.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -85,7 +84,7 @@ static filebrowser_filter_t __fb_filters[] = {
  /* subtitles */
   { NULL, ".sub .srt .asc .smi .ssa "                                              },
   /* playlists */
-  { NULL, ".pls .m3u .sfv .tox .asx .smil .smi .xml "                              },
+  { NULL, ".pls .m3u .sfv .tox .asx .smil .smi .xml .fxd "                         },
   { "*.4xm", ".4xm "                                                               },
   { "*.ac3", ".ac3 "                                                               },
   { "*.aif, *.aiff,", ".aif .aiff "                                                },
@@ -142,8 +141,6 @@ struct filebrowser_s {
 
   xitk_widget_t                  *origin;
 
-  Cursor                          cursor[2];
-
   xitk_widget_t                  *directories_browser;
   xitk_widget_t                  *directories_sort;
   int                             directories_sort_direction;
@@ -182,6 +179,8 @@ struct filebrowser_s {
   
   xitk_widget_t                  *close;
 
+  hidden_file_toggle_t            hidden_cb;
+
   filebrowser_callback_button_t   cbb[3];
   xitk_widget_t                  *cb_buttons[2];
   
@@ -235,10 +234,10 @@ static void fb_reactivate(filebrowser_t *fb) {
 
 static void _fb_set_cursor(filebrowser_t *fb, int state) {
   if(fb) {
-    XLockDisplay(gGui->display);
-    XDefineCursor(gGui->display, (xitk_window_get_window(fb->xwin)), fb->cursor[state]);
-    XSync(gGui->display, False);
-    XUnlockDisplay(gGui->display);
+    if(state == WAIT_CURS)
+      xitk_cursors_define_window_cursor(gGui->display, (xitk_window_get_window(fb->xwin)), xitk_cursor_watch);
+    else
+      xitk_cursors_restore_window_cursor(gGui->display, (xitk_window_get_window(fb->xwin)));
   }
 }
 /*
@@ -384,7 +383,7 @@ static void fb_create_input_window(char *title, char *text,
 
   {
     char buffer[256];
-    sprintf(buffer, "filenameed%d", (unsigned int) time(NULL));
+    snprintf(buffer, sizeof(buffer), "filenameed%d", (unsigned int) time(NULL));
     
     fne->widget_key = xitk_register_event_handler(buffer, 
 						  (xitk_window_get_window(fne->xwin)),
@@ -398,7 +397,7 @@ static void fb_create_input_window(char *title, char *text,
   XLockDisplay(gGui->display);
   XRaiseWindow(gGui->display, xitk_window_get_window(fne->xwin));
   XMapWindow(gGui->display, xitk_window_get_window(fne->xwin));
-  if(!gGui->use_root_window)
+  if(!gGui->use_root_window && gGui->video_display == gGui->display)
     XSetTransientForHint(gGui->display, 
 			 xitk_window_get_window(fne->xwin), gGui->video_window);
   XUnlockDisplay(gGui->display);
@@ -425,9 +424,17 @@ static int is_file_match_to_filter(filebrowser_t *fb, char *file) {
     char *ending;
 
     if((ending = strrchr(file, '.'))) {
-      char ext[strlen(ending) + 2];
+      char  ext[strlen(ending) + 2];
+      char *p;
       
       sprintf(ext, "%s ", ending);
+
+      p = ext;
+      while(*p && (*p != '\0')) {
+	*p = tolower(*p);
+	p++;
+      }
+	
       if(strstr(__fb_filters[fb->filter_selected].ending, ext))
 	return 1;
       
@@ -441,15 +448,15 @@ static void fb_update_origin(filebrowser_t *fb) {
   
   if(strcasecmp(fb->current_dir, "/")) {
     if(strlen(fb->filename))
-      sprintf(buf, "%s/%s", fb->current_dir, fb->filename);
+      snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->filename);
     else
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   }
   else {
     if(strlen(fb->filename))
-      sprintf(buf, "%s%s", fb->current_dir, fb->filename);
+      snprintf(buf, sizeof(buf), "%s%s", fb->current_dir, fb->filename);
     else
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   }
   
   xitk_inputtext_change_text(fb->origin, buf); 
@@ -481,15 +488,15 @@ static void fb_extract_path_and_file(filebrowser_t *fb, char *filepathname) {
     
     if(filename && (strlen(filename) > 1)) {
       *filename++ = '\0';
-      sprintf(fb->filename, "%s", filename);
+      snprintf(fb->filename, sizeof(fb->filename), "%s", filename);
       
       if(is_a_dir(_filepathname))
-	sprintf(fb->current_dir, "%s", _filepathname);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", _filepathname);
 
     }
     else {
       if(is_a_dir((char *)_filepathname)) {
-	sprintf(fb->current_dir, "%s", _filepathname);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", _filepathname);
 	memset(&fb->filename, 0, sizeof(fb->filename));
       }
     }
@@ -682,7 +689,7 @@ static void fb_getdir(filebrowser_t *fb) {
   while((pdirent = readdir(pdir)) != NULL) {
     
     memset(&fullfilename, 0, sizeof(fullfilename));
-    sprintf(fullfilename, "%s/%s", fb->current_dir, pdirent->d_name);
+    snprintf(fullfilename, sizeof(fullfilename), "%s/%s", fb->current_dir, pdirent->d_name);
     
     if(is_a_dir(fullfilename)) {
       
@@ -743,7 +750,7 @@ static void fb_select(xitk_widget_t *w, void *data, int selected) {
   filebrowser_t *fb = (filebrowser_t *) data;
   
   if(w == fb->files_browser) {
-    sprintf(fb->filename, "%s", fb->norm_files[selected].name);
+    snprintf(fb->filename, sizeof(fb->filename), "%s", fb->norm_files[selected].name);
     fb_update_origin(fb);
   }
 }
@@ -780,7 +787,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
       char *p;
       
       memset(&buf, '\0', sizeof(buf));
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
       if(strlen(buf) > 1) { /* not '/' directory */
 	
 	p = &buf[strlen(buf)-1];
@@ -793,7 +800,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
 	if((strlen(buf) > 1) && *p == '/') 
 	  *p = '\0';
 	
-	sprintf(fb->current_dir, "%s", buf);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", buf);
       }
     }
     else {
@@ -801,15 +808,15 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
       /* not '/' directory */
       if(strcasecmp(fb->current_dir, "/")) {
 	memset(&buf, 0, sizeof(buf));
-	sprintf(buf, "%s/%s", fb->current_dir, fb->dir_files[selected].name);
+	snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->dir_files[selected].name);
       }
       else {
 	memset(&buf, 0, sizeof(buf));
-	sprintf(buf, "/%s", fb->dir_files[selected].name);
+	snprintf(buf, sizeof(buf), "/%s", fb->dir_files[selected].name);
       }
       
       if(is_a_dir(buf))
-	sprintf(fb->current_dir, "%s", buf);
+	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", buf);
 
     }
     
@@ -817,7 +824,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
     fb_getdir(fb);
   }
   else if(w == fb->files_browser) {
-    sprintf(fb->filename, "%s", fb->norm_files[selected].name);
+    snprintf(fb->filename, sizeof(fb->filename), "%s", fb->norm_files[selected].name);
     fb_callback_button_cb(fb->cb_buttons[0], (void *)data);
   }
 
@@ -829,7 +836,7 @@ static void fb_change_origin(xitk_widget_t *w, void *data, char *currenttext) {
   char          *p;
 
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s", currenttext);
+  snprintf(buf, sizeof(buf), "%s", currenttext);
 
   p = &buf[strlen(buf) - 1];
   while((strlen(buf) > 1) && (*p == '/'))
@@ -901,8 +908,6 @@ static void fb_exit(xitk_widget_t *w, void *data) {
     xitk_list_free((XITK_WIDGET_LIST_LIST(fb->widget_list)));
     
     XLockDisplay(gGui->display);
-    XFreeCursor(gGui->display, fb->cursor[NORMAL_CURS]);
-    XFreeCursor(gGui->display, fb->cursor[WAIT_CURS]);
     XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(fb->widget_list)));
     XUnlockDisplay(gGui->display);
     
@@ -975,9 +980,9 @@ static void fb_delete_file_cb(xitk_widget_t *w, void *data, int button) {
       int sel = xitk_browser_get_current_selected(fb->files_browser);
 
       memset(&buf, 0, sizeof(buf));
-      sprintf(buf, "%s", fb->current_dir);
+      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
       if(strlen(fb->current_dir) > 1)
-	sprintf(buf, "%s/", buf);
+	strcat(buf, "/");
       sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
       
       if((unlink(buf)) == -1)
@@ -998,9 +1003,9 @@ static void fb_delete_file(xitk_widget_t *w, void *data) {
   if((sel = xitk_browser_get_current_selected(fb->files_browser)) >= 0) {
     char buf[256 + XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
-    sprintf(buf, "Do you really want to delete the file: '%s", fb->current_dir);
+    snprintf(buf, sizeof(buf), _("Do you really want to delete the file: '%s'"), fb->current_dir);
     if(strlen(fb->current_dir) > 1)
-      sprintf(buf, "%s/", buf);
+      strcat(buf, "/");
     sprintf(buf, "%s%s' ?.", buf, fb->norm_files[sel].name);
     
     fb_deactivate(fb);
@@ -1017,9 +1022,9 @@ static void fb_rename_file_cb(xitk_widget_t *w, void *data, char *newname) {
   int sel = xitk_browser_get_current_selected(fb->files_browser);
   
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s", fb->current_dir);
+  snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   if(strlen(fb->current_dir) > 1)
-    sprintf(buf, "%s/", buf);
+    strcat(buf, "/");
   sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
   
   if((rename(buf, newname)) == -1)
@@ -1036,9 +1041,9 @@ static void fb_rename_file(xitk_widget_t *w, void *data) {
     char buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
     memset(&buf, 0, sizeof(buf));
-    sprintf(buf, "%s", fb->current_dir);
+    snprintf(buf, sizeof(buf), "%s", fb->current_dir);
     if(strlen(fb->current_dir) > 1)
-      sprintf(buf, "%s/", buf);
+      strcat(buf, "/");
     sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
     
     fb_deactivate(fb);
@@ -1059,9 +1064,9 @@ static void fb_create_directory(xitk_widget_t *w, void *data) {
   char           buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, "%s", fb->current_dir);
+  snprintf(buf, sizeof(buf), "%s", fb->current_dir);
   if(strlen(fb->current_dir) > 1)
-    sprintf(buf, "%s/", buf);
+    strcat(buf, "/");
   
   fb_deactivate(fb);
   fb_create_input_window(_("Create a new directory"), buf, fb_create_directory_cb, fb);
@@ -1078,6 +1083,7 @@ static void fb_hidden_files(xitk_widget_t *w, void *data, int state) {
   filebrowser_t *fb = (filebrowser_t *) data;
   
   fb->show_hidden_files = state;
+  fb->hidden_cb(1, state);
   fb_getdir(fb);
 }
 static void fb_lbl_hidden_files(xitk_widget_t *w, void *data) {
@@ -1157,7 +1163,7 @@ char **filebrowser_get_all_files(filebrowser_t *fb) {
   return files;
 }
 
-filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
+filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden_file_toggle_t hidden_cb,
 				  filebrowser_callback_button_t *cbb1,
 				  filebrowser_callback_button_t *cbb2,
 				  filebrowser_callback_button_t *cbb_close) {
@@ -1220,9 +1226,10 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   fb->files_num                  = 0;
   fb->directories_sort_direction = DEFAULT_SORT;
   fb->files_sort_direction       = DEFAULT_SORT;
-  fb->show_hidden_files          = 1;
+  fb->hidden_cb                  = hidden_cb;
+  fb->show_hidden_files          = hidden_cb(0, 0);
 
-  sprintf(fb->current_dir, "%s", xine_get_homedir());
+  snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", xine_get_homedir());
   memset(&fb->filename, 0, sizeof(fb->filename));
   fb_extract_path_and_file(fb, filepathname);
 
@@ -1247,8 +1254,6 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
   XLockDisplay(gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(fb->xwin)), None, None);
-  fb->cursor[NORMAL_CURS] = XCreateFontCursor(gGui->display, XC_left_ptr);
-  fb->cursor[WAIT_CURS] = XCreateFontCursor(gGui->display, XC_watch);
   XUnlockDisplay(gGui->display);
 
   fb->widget_list                = xitk_widget_list_new();
@@ -1632,7 +1637,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
 
   {
     char buffer[256];
-    sprintf(buffer, "filebrowser%d", (unsigned int) time(NULL));
+    snprintf(buffer, sizeof(buffer), "filebrowser%d", (unsigned int) time(NULL));
     fb->widget_key = xitk_register_event_handler(buffer, 
 						 (xitk_window_get_window(fb->xwin)),
 						 fb_handle_events,
