@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.204 2004/04/13 19:55:48 f1rmb Exp $
+ * $Id: panel.c,v 1.217 2004/12/15 21:10:45 f1rmb Exp $
  *
  * xine panel related stuff
  *
@@ -39,7 +39,7 @@
 
 extern gGui_t     *gGui;
 
-_panel_t          *panel;
+_panel_t          *panel = NULL;
 
 void panel_update_nextprev_tips(void) {
 
@@ -88,13 +88,17 @@ void panel_show_tips(void) {
   playlist_show_tips(panel->tips.enable, panel->tips.timeout);
   control_show_tips(panel->tips.enable, panel->tips.timeout);
   mrl_browser_show_tips(panel->tips.enable, panel->tips.timeout);
+  event_sender_show_tips(panel->tips.enable, panel->tips.timeout);
+  mmk_editor_show_tips(panel->tips.enable, panel->tips.timeout);
+  setup_show_tips(panel->tips.enable, panel->tips.timeout);
 }
 
+/* Somewhat paranoia conditionals (Hans, YOU're paranoid ;-) )*/
 int panel_get_tips_enable(void) {
-  return panel->tips.enable;
+  return (panel) ? panel->tips.enable : 0;
 }
 unsigned long panel_get_tips_timeout(void) {
-  return panel->tips.timeout;
+  return (panel) ? panel->tips.timeout : 0;
 }
 
 /*
@@ -105,8 +109,15 @@ static void panel_enable_tips_cb(void *data, xine_cfg_entry_t *cfg) {
   panel_show_tips();
 }
 static void panel_timeout_tips_cb(void *data, xine_cfg_entry_t *cfg) {
-  panel->tips.timeout = cfg->num_value;
+  panel->tips.timeout = (unsigned long) cfg->num_value;
+
   xitk_set_widgets_tips_timeout(panel->widget_list, panel->tips.timeout);
+  control_update_tips_timeout(panel->tips.timeout);
+  event_sender_update_tips_timeout(panel->tips.timeout);
+  mmk_editor_update_tips_timeout(panel->tips.timeout);
+  mrl_browser_update_tips_timeout(panel->tips.timeout);
+  playlist_update_tips_timeout(panel->tips.timeout);
+  setup_update_tips_timeout(panel->tips.timeout);
 }
 
 /*
@@ -132,10 +143,50 @@ static void panel_store_new_position(int x, int y, int w, int h) {
 
 }
 
+static void panel_exit(xitk_widget_t *w, void *data) {
+
+  if(panel) {
+    window_info_t wi;
+    
+    panel->visible = 0;
+
+    if((xitk_get_window_info(panel->widget_key, &wi))) {
+      config_update_num ("gui.panel_x", wi.x);
+      config_update_num ("gui.panel_y", wi.y);
+      WINDOW_INFO_ZERO(&wi);
+    }
+
+    xitk_unregister_event_handler(&panel->widget_key);
+
+    XLockDisplay(gGui->display);
+    XUnmapWindow(gGui->display, gGui->panel_window);
+    XUnlockDisplay(gGui->display);
+
+    xitk_destroy_widgets(panel->widget_list);
+
+    XLockDisplay(gGui->display);
+    XDestroyWindow(gGui->display, gGui->panel_window);
+    Imlib_destroy_image(gGui->imlib_data, panel->bg_image);
+    XUnlockDisplay(gGui->display);
+
+    gGui->panel_window = None;
+    xitk_list_free((XITK_WIDGET_LIST_LIST(panel->widget_list)));
+
+    XLockDisplay(gGui->display);
+    XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(panel->widget_list)));
+    XUnlockDisplay(gGui->display);
+
+    free(panel->widget_list);
+    
+    free(panel);
+    panel = NULL;  
+  }
+}
+
 /*
  * Change the current skin.
  */
-void panel_change_skins(void) {
+void panel_change_skins(int synthetic) {
   ImlibImage   *new_img, *old_img;
   XSizeHints    hint;
   
@@ -173,7 +224,7 @@ void panel_change_skins(void) {
 
   XLockDisplay(gGui->display);
 
-  if(!gGui->use_root_window)
+  if(!gGui->use_root_window && gGui->video_display == gGui->display)
     XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
 
   Imlib_destroy_image(gGui->imlib_data, old_img);
@@ -265,25 +316,46 @@ void panel_update_runtime_display(void) {
   }
 
   if(pos || seconds) {
-    remain = (length - seconds) / 1000;
+    int days, rdays;
+    
+    remain  = (length - seconds) / 1000;
     seconds /= 1000;
-    
-    if(panel->runtime_mode == 0)
-      sprintf(timestr, "%02d:%02d:%02d", seconds / (60*60), (seconds / 60) % 60, seconds % 60);
+    days    = seconds / (60*60*24);
+    rdays   = remain / (60*60*24);
+
+    if(panel->runtime_mode == 0) {
+      if(days > 0)
+	snprintf(timestr, sizeof(timestr), "% 4d::%02d", days, seconds / (60*60));
+      else
+	snprintf(timestr, sizeof(timestr), "%02d:%02d:%02d", seconds / (60*60), (seconds / 60) % 60, seconds % 60);
+    }
     else
-      sprintf(timestr, "%02d:%02d:%02d", remain / (60*60), (remain / 60) % 60, remain % 60);
+      if(remain < 0)
+	snprintf(timestr, sizeof(timestr), "%s", "  :  :  ");
+      else {
+	if(rdays > 0)
+	  snprintf(timestr, sizeof(timestr), "% 4d::%02d", rdays, remain / (60*60));
+	else
+	  snprintf(timestr, sizeof(timestr), "%02d:%02d:%02d", remain / (60*60), (remain / 60) % 60, remain % 60);
+      }
     
   }
   else
-    sprintf(timestr, "%s", "--:--:--");
+    snprintf(timestr, sizeof(timestr), "%s", "--:--:--");
   
-  if(length) {
+  if(length > 0) {
+    int days;
+
     length /= 1000;
-    sprintf(buffer, _("Total time: %02d:%02d:%02d"), 
-	    length / (60*60), (length / 60) % 60, length % 60);
+    days = length / (60*60*24);
+    
+    if(days > 0)
+      snprintf(buffer, sizeof(buffer), "%s% 4d::%02d", _("Total time: "), days, length / (60*60));
+    else
+      snprintf(buffer, sizeof(buffer), "%s%02d:%02d:%02d", _("Total time: "), length / (60*60), (length / 60) % 60, length % 60);
   }
   else
-    sprintf(buffer, "%s", _("Total time: --:--:--"));
+    snprintf(buffer, sizeof(buffer), "%s%s", _("Total time: "), "??:??:??");
   
   xitk_set_widget_tips(panel->runtime_label, buffer);
   
@@ -347,72 +419,89 @@ static void *slider_loop(void *dummy) {
       
       if(!(i % 2)) {
 	osd_update();
-
+	
 	if(gGui->mrl_overrided) {
 	  gGui->mrl_overrided--;
 	  if(gGui->mrl_overrided == 0)
 	    panel_update_mrl_display();
 	}
       }
-
+      
       if((status == XINE_STATUS_PLAY) && (speed != XINE_SPEED_PAUSE)) {
+	char *ident = stream_infos_get_ident_from_stream(gGui->stream);
 	
-	if(gGui->ssaver_timeout) {
-	  
-	  if(!(i % 2))
-	    screensaver_timer++;
-	  
-	  if(screensaver_timer >= gGui->ssaver_timeout) {
-	    screensaver_timer = 0;
-	    video_window_reset_ssaver();
+	if(ident) {
+	  if(strcmp(gGui->mmk.ident, ident)) {
+	    if(gGui->mmk.ident)
+	      free(gGui->mmk.ident);
+	    if(gGui->playlist.mmk[gGui->playlist.cur]->ident)
+	      free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
 	    
+	    gGui->mmk.ident = strdup(ident);
+	    gGui->playlist.mmk[gGui->playlist.cur]->ident = strdup(ident);
+	    
+	    video_window_set_mrl(gGui->mmk.ident);
+	    playlist_mrlident_toggle();
+	    panel_update_mrl_display();
 	  }
-	}  
-
-	if(gGui->logo_mode == 0) {
+	  free(ident);
+	}
+	else
+	  video_window_set_mrl((char *)gGui->mmk.mrl);
+	
+	if(!xitk_is_window_iconified(gGui->video_display, gGui->video_window)) {
 	  
-	  if(panel_is_visible()) {
+	  if(gGui->ssaver_timeout) {
 	    
-	    panel_update_runtime_display();
-
-	    if(xitk_is_widget_enabled(panel->playback_widgets.slider_play)) {
-	      if(pos >= 0)
-		xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
+	    if(!(i % 2))
+	      screensaver_timer++;
+	    
+	    if(screensaver_timer >= gGui->ssaver_timeout) {
+	      screensaver_timer = 0;
+	      video_window_reset_ssaver();
+	      
 	    }
+	  }  
+	  
+	  if(gGui->logo_mode == 0) {
 	    
-	    if(!(i % 20)) {
-	      int level = 0;
+	    if(panel_is_visible()) {
 	      
-	      panel_update_channel_display();
+	      panel_update_runtime_display();
 	      
-	      if((gGui->mixer.method == SOUND_CARD_MIXER) &&
-		 (gGui->mixer.caps & MIXER_CAP_VOL)) {
-		level = gGui->mixer.volume_level = 
-		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
-	      }
-	      else if(gGui->mixer.method == SOFTWARE_MIXER) {
-		level = gGui->mixer.amp_level = 
-		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+	      if(xitk_is_widget_enabled(panel->playback_widgets.slider_play)) {
+		if(pos >= 0)
+		  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
 	      }
 	      
-	      xitk_slider_set_pos(panel->mixer.slider, level);
-	      panel_check_mute();
+	      if(!(i % 20)) {
+		int level = 0;
+		
+		panel_update_channel_display();
+		
+		if((gGui->mixer.method == SOUND_CARD_MIXER) &&
+		   (gGui->mixer.caps & MIXER_CAP_VOL)) {
+		  level = gGui->mixer.volume_level = 
+		    xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
+		}
+		else if(gGui->mixer.method == SOFTWARE_MIXER) {
+		  level = gGui->mixer.amp_level = 
+		    xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+		}
+		
+		xitk_slider_set_pos(panel->mixer.slider, level);
+		panel_check_mute();
+		
+		i = 0;
+	      }
 	      
-	      i = 0;
 	    }
 	    
+	    if(stream_infos_is_visible() && gGui->stream_info_auto_update)
+	      stream_infos_update_infos();
 	  }
-	  
-	  if(stream_infos_is_visible() && gGui->stream_info_auto_update)
-	    stream_infos_update_infos();
 	}
       }
-      
-      if(gGui->got_reference_stream) {
-	gGui->got_reference_stream--;
-	gui_play(NULL, NULL);
-      }
-
     }
     
     if(gGui->cursor_visible) {
@@ -446,7 +535,7 @@ int panel_is_visible(void) {
     if(gGui->use_root_window)
       return xitk_is_window_visible(gGui->display, gGui->panel_window);
     else
-      return panel->visible;
+      return panel->visible && xitk_is_window_visible(gGui->display, gGui->panel_window);
   }
 
   return 0;
@@ -458,90 +547,46 @@ int panel_is_visible(void) {
 static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
   int visible = xitk_is_window_visible(gGui->display, gGui->panel_window);
 
-  if(!panel->visible && playlist_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && playlist_is_visible()))
-      playlist_toggle_visibility(NULL, NULL);
-  }
-    
-  if(!panel->visible && control_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && control_is_visible()))
-      control_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !playlist_is_visible()) || (visible && playlist_is_visible()))
+    playlist_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && mrl_browser_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && mrl_browser_is_visible()))
-      mrl_browser_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !control_is_visible()) || (visible && control_is_visible()))
+    control_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && setup_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && setup_is_visible()))
-      setup_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !mrl_browser_is_visible()) || (visible && mrl_browser_is_visible()))
+    mrl_browser_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && viewlog_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && viewlog_is_visible()))
-      viewlog_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !setup_is_visible()) || (visible && setup_is_visible()))
+    setup_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && kbedit_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && kbedit_is_visible()))
-      kbedit_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !viewlog_is_visible()) || (visible && viewlog_is_visible()))
+    viewlog_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && event_sender_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && event_sender_is_visible()))
-      event_sender_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !kbedit_is_visible()) || (visible && kbedit_is_visible()))
+    kbedit_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && stream_infos_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && stream_infos_is_visible()))
-      stream_infos_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !event_sender_is_visible()) || (visible && event_sender_is_visible()))
+    event_sender_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && tvset_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && tvset_is_visible()))
-      tvset_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !stream_infos_is_visible()) || (visible && stream_infos_is_visible()))
+    stream_infos_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && pplugin_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && pplugin_is_visible()))
-      pplugin_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !tvset_is_visible()) || (visible && tvset_is_visible()))
+    tvset_toggle_visibility(NULL, NULL);
 
-  if(!panel->visible && help_is_visible()) {
-  }
-  else {
-    if(!gGui->use_root_window || (gGui->use_root_window && visible && help_is_visible()))
-      help_toggle_visibility(NULL, NULL);
-  }
+  if(((!panel->visible || !visible) && !pplugin_is_visible()) || (visible && pplugin_is_visible()))
+    pplugin_toggle_visibility(NULL, NULL);
 
-  if (panel->visible) {
+  if(((!panel->visible || !visible) && !help_is_visible()) || (visible && help_is_visible()))
+    help_toggle_visibility(NULL, NULL);
+
+  if (panel->visible && gGui->video_display == gGui->display) {
     
     XLockDisplay(gGui->display);
     
     if (video_window_is_visible ()) {
       if(gGui->use_root_window) { /* Using root window */
-	if(xitk_is_window_visible(gGui->display, gGui->panel_window))
+	if(visible)
 	  XIconifyWindow(gGui->display, gGui->panel_window, gGui->screen);
 	else
 	  XMapWindow(gGui->display, gGui->panel_window);
@@ -553,14 +598,19 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
 	xitk_hide_widgets(panel->widget_list);
       }
     }
-    else
-      XIconifyWindow(gGui->display, gGui->panel_window, (XDefaultScreen(gGui->display)));
-
+    else {
+      if(visible)
+	XIconifyWindow(gGui->display, gGui->panel_window, (XDefaultScreen(gGui->display)));
+      else
+	XMapWindow(gGui->display, gGui->panel_window);
+    }
+    
     if(gGui->cursor_grabbed)
-       XGrabPointer(gGui->display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
+      XGrabPointer(gGui->video_display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
     
     /* Give focus to video output window */
     try_to_set_input_focus(gGui->video_window);
+    
     XUnlockDisplay(gGui->display);
      
   }
@@ -573,7 +623,7 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
     
     XRaiseWindow(gGui->display, gGui->panel_window); 
     XMapWindow(gGui->display, gGui->panel_window); 
-    if(!gGui->use_root_window)
+    if(!gGui->use_root_window && gGui->video_display == gGui->display)
       XSetTransientForHint (gGui->display,  gGui->panel_window, gGui->video_window);
     XUnlockDisplay (gGui->display);
 
@@ -705,7 +755,7 @@ void panel_update_channel_display (void) {
 
   default:
     if(!xine_get_audio_lang (gGui->stream, channel, &buffer[0]))
-      sprintf(buffer, "%3d", channel);
+      snprintf(buffer, sizeof(buffer), "%3d", channel);
     lang = buffer;
     break;
   }
@@ -727,7 +777,7 @@ void panel_update_channel_display (void) {
 
   default:
     if(!xine_get_spu_lang (gGui->stream, channel, &buffer[0]))
-      sprintf(buffer, "%3d", channel);
+      snprintf(buffer, sizeof(buffer), "%3d", channel);
     lang = buffer;
     break;
   }
@@ -830,7 +880,7 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 /*
  * Handle X events here.
  */
-void panel_handle_event(XEvent *event, void *data) {
+static void panel_handle_event(XEvent *event, void *data) {
 
   switch(event->type) {
   case ButtonPress:
@@ -890,8 +940,9 @@ void panel_add_autoplay_buttons(void) {
 				(panel->autoplay_plugins[i] =
 				 xitk_labelbutton_create (panel->widget_list, 
 							  gGui->skin_config, &lb)));
-      xitk_set_widget_tips(panel->autoplay_plugins[i], 
-			   (char *) xine_get_input_plugin_description(gGui->xine, autoplay_label));
+      xitk_set_widget_tips_and_timeout(panel->autoplay_plugins[i],
+				       (char *) xine_get_input_plugin_description(gGui->xine, autoplay_label),
+				       panel->tips.timeout);
       
       if(!panel->tips.enable)
 	xitk_disable_widget_tips(panel->autoplay_plugins[i]);
@@ -965,9 +1016,9 @@ void panel_add_mixer_control(void) {
 
   /* Tips should be available only if widgets are enabled */
   if(gGui->mixer.caps & MIXER_CAP_VOL)
-    xitk_set_widget_tips(panel->mixer.slider, _("Volume control"));
+    xitk_set_widget_tips_and_timeout(panel->mixer.slider, _("Volume control"), panel->tips.timeout);
   if(gGui->mixer.caps & MIXER_CAP_MUTE)
-    xitk_set_widget_tips(panel->mixer.mute, _("Mute toggle"));
+    xitk_set_widget_tips_and_timeout(panel->mixer.mute, _("Mute toggle"), panel->tips.timeout);
 
   if(!panel->tips.enable) {
     xitk_disable_widget_tips(panel->mixer.slider);
@@ -979,7 +1030,7 @@ void panel_add_mixer_control(void) {
 void panel_deinit(void) {
   if(panel_is_visible())
     _panel_toggle_visibility(NULL, NULL);
-  xitk_unregister_event_handler(&panel->widget_key);
+  panel_exit(NULL, NULL);
 }
 
 void panel_reparent(void) {
@@ -1112,7 +1163,7 @@ void panel_init (void) {
                   PropModeReplace, (unsigned char *) &mwmhints,
                   PROP_MWM_HINTS_ELEMENTS);
   
-  if(!gGui->use_root_window)
+  if(!gGui->use_root_window && gGui->video_display == gGui->display)
     XSetTransientForHint (gGui->display, gGui->panel_window, gGui->video_window);
 
   /* 
@@ -1408,28 +1459,31 @@ void panel_init (void) {
 						 CONFIG_LEVEL_ADV,
 						 panel_enable_tips_cb,
 						 CONFIG_NO_DATA);
-  panel->tips.timeout = xine_config_register_num(gGui->xine, "gui.tips_timeout", 
-						 500,
-						 _("tips timeout (ms)"), 
-						 CONFIG_NO_HELP,
-						 CONFIG_LEVEL_ADV,
-						 panel_timeout_tips_cb, 
-						 CONFIG_NO_DATA);
+  panel->tips.timeout = (unsigned long) xine_config_register_num(gGui->xine, "gui.tips_timeout", 
+								 500,
+								 _("tips timeout (ms)"), 
+								 CONFIG_NO_HELP,
+								 CONFIG_LEVEL_ADV,
+								 panel_timeout_tips_cb, 
+								 CONFIG_NO_DATA);
 
   panel_update_mrl_display();
   panel_update_nextprev_tips();
-  panel_show_tips();
 
   /* 
    * show panel 
    */
-  panel->visible = xine_config_register_bool (gGui->xine, "gui.panel_visible", 
-					      1,
-					      _("gui panel visibility"),
-					      CONFIG_NO_HELP,
-					      CONFIG_LEVEL_DEB,
-					      CONFIG_NO_CB,
-					      CONFIG_NO_DATA);
+  {
+    int visible = xine_config_register_bool(gGui->xine, "gui.panel_visible", 
+					    1,
+					    _("gui panel visibility"),
+					    CONFIG_NO_HELP,
+					    CONFIG_LEVEL_DEB,
+					    CONFIG_NO_CB,
+					    CONFIG_NO_DATA);
+
+      panel->visible = (!gGui->no_gui) ? visible : 0;
+  }
   
   /*  The user don't want panel on startup */
   if(panel->visible && (actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_VISIBLITY))) {
@@ -1437,7 +1491,7 @@ void panel_init (void) {
     config_update_num ("gui.panel_visible", panel->visible);
   }
   
-  if(gGui->use_root_window && (!panel->visible))
+  if((gGui->use_root_window || gGui->video_display != gGui->display) && (!panel->visible))
     panel->visible = 1;
   
   if (panel->visible) {
@@ -1463,14 +1517,14 @@ void panel_init (void) {
   
   {
     pthread_attr_t       pth_attrs;
-#if !defined (__OpenBSD__)
+#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
     struct sched_param   pth_params;
 #endif
     
     pthread_attr_init(&pth_attrs);
 
     /* this won't work on linux, freebsd 5.0 */
-#if !defined (__OpenBSD__)
+#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
     pthread_attr_getschedparam(&pth_attrs, &pth_params);
     pth_params.sched_priority = sched_get_priority_min(SCHED_OTHER);
     pthread_attr_setschedparam(&pth_attrs, &pth_params);
