@@ -1,6 +1,6 @@
 #warning IMPLEMENT POST SUPPORT
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: network.c,v 1.48 2003/07/22 20:27:31 f1rmb Exp $
+ * $Id: network.c,v 1.64 2004/08/24 20:06:26 f1rmb Exp $
  *
  * xine network related stuff
  *
@@ -37,6 +37,8 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_READLINE
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -64,10 +66,10 @@
 #include <netdb.h>
 #include <pthread.h>
 
-#include "common.h"
+#include <readline.h>
+#include <history.h>
 
-#include "readline.h"
-#include "history.h"
+#include "common.h"
 
 #ifndef	MSG_NOSIGNAL
 #define	MSG_NOSIGNAL	     0
@@ -160,7 +162,6 @@ typedef struct client_info_s client_info_t;
 typedef struct passwds_s passwds_t;
 typedef void (*cmd_func_t)(commands_t *, client_info_t *);
 
-static pthread_t     thread_server;
 static passwds_t   **passwds = NULL;
 
 static void do_commands(commands_t *, client_info_t *);
@@ -174,7 +175,9 @@ static void do_stop(commands_t *, client_info_t *);
 static void do_pause(commands_t *, client_info_t *);
 static void do_exit(commands_t *, client_info_t *);
 static void do_fullscreen(commands_t *, client_info_t *);
+#ifdef HAVE_XINERAMA
 static void do_xinerama_fullscreen(commands_t *, client_info_t *);
+#endif
 static void do_get(commands_t *, client_info_t *);
 static void do_set(commands_t *, client_info_t *);
 static void do_gui(commands_t *, client_info_t *);
@@ -282,8 +285,12 @@ static commands_t commands[] = {
     "  playlist select <num>\n"
     "  playlist delete all|*\n"
     "  playlist delete <num>\n"
+    "  playlist first\n"
+    "  playlist prev\n"
     "  playlist next\n"
-    "  playlist prev"
+    "  playlist last\n"
+    "  playlist stop\n"
+    "  playlist contine"
   },
   { "stop",        NO_ARGS,         PUBLIC,          NEED_AUTH,     do_stop,
     "stop playback", 
@@ -301,10 +308,12 @@ static commands_t commands[] = {
     "fullscreen toggle", 
     "  fullscreen"
   },
-  { "xineramafull" ,  NO_ARGS,         PUBLIC,          NEED_AUTH,     do_xinerama_fullscreen,
+#ifdef HAVE_XINERAMA
+  { "xineramafull" ,  NO_ARGS,      PUBLIC,          NEED_AUTH,     do_xinerama_fullscreen,
     "xinerama fullscreen toggle",
     "  expand display on further screens"
   },
+#endif
   { "get",         REQUIRE_ARGS,    PUBLIC,          NEED_AUTH,     do_get,
     "get values", 
     "  get status\n"
@@ -314,6 +323,8 @@ static commands_t commands[] = {
     "  get audio mute\n"
     "  get spu channel\n"
     "  get spu lang\n"
+    "  get spu offset\n"
+    "  get av offset\n"
     "  get speed\n"
     "  get position\n"
     "  get length\n"
@@ -325,6 +336,8 @@ static commands_t commands[] = {
     "  set audio volume <%>\n"
     "  set audio mute <state>\n"
     "  set spu channel <num>\n"
+    "  set spu offset <offset>\n"
+    "  set av offset <offset>\n"
     "  set speed <XINE_SPEED_PAUSE|XINE_SPEED_SLOW_4|XINE_SPEED_SLOW_2|XINE_SPEED_NORMAL|XINE_SPEED_FAST_2|XINE_SPEED_FAST_4>\n"
     "            <        |       |        /4       |        /2       |        =        |        *2       |        *4       >\n"
     "  set loop <no | loop | repeat | shuffle | shuffle+>"
@@ -338,6 +351,8 @@ static commands_t commands[] = {
     "  gui control\n"
     "  gui mrl\n"
     "  gui setup\n"
+    "  gui vpost\n"
+    "  gui help\n"
     "  gui log"
   },
   { "event",       REQUIRE_ARGS,    PUBLIC,          NEED_AUTH,     do_event,
@@ -389,11 +404,35 @@ static commands_t commands[] = {
   } 
 };
 
+static struct {
+  char        *name;
+  int          status;
+} status_struct[] = {
+  { "XINE_STATUS_IDLE"   , XINE_STATUS_IDLE },
+  { "XINE_STATUS_PLAY"   , XINE_STATUS_PLAY },
+  { "XINE_STATUS_STOP"   , XINE_STATUS_STOP },
+  { "XINE_STATUS_QUIT"   , XINE_STATUS_QUIT },
+  { NULL                 , -1               }
+};
+
+static struct {
+  char        *name;
+  int          speed;
+} speeds_struct[] = {
+  { "XINE_SPEED_PAUSE"   , XINE_SPEED_PAUSE  },
+  { "XINE_SPEED_SLOW_4"  , XINE_SPEED_SLOW_4 },
+  { "XINE_SPEED_SLOW_2"  , XINE_SPEED_SLOW_2 },
+  { "XINE_SPEED_NORMAL"  , XINE_SPEED_NORMAL },
+  { "XINE_SPEED_FAST_2"  , XINE_SPEED_FAST_2 },
+  { "XINE_SPEED_FAST_4"  , XINE_SPEED_FAST_4 },
+  { NULL                 , 0                 }
+};
+
 extern gGui_t *gGui;
 
 #endif
 
-void sock_err(const char *error_msg, ...) {
+static void sock_err(const char *error_msg, ...) {
   va_list args;
   
   va_start(args, error_msg);
@@ -401,7 +440,7 @@ void sock_err(const char *error_msg, ...) {
   va_end(args);
 }
 
-int sock_create(const char *service, const char *transport, struct sockaddr_in *sin) {
+static int sock_create(const char *service, const char *transport, struct sockaddr_in *sin) {
   struct servent    *iservice;
   struct protoent *itransport;
   int                sock;
@@ -444,57 +483,10 @@ int sock_create(const char *service, const char *transport, struct sockaddr_in *
   return sock;
 }
 
-int sock_client(const char *host, const char *service, const char *transport) {
-  struct sockaddr_in   sin;
-  struct hostent      *ihost;
-  int                  sock;
-  
-  sock = sock_create(service, transport, &sin);
-
-  if ((sin.sin_addr.s_addr = inet_addr(host)) == -1) {
-    ihost = gethostbyname(host);
-    
-    if(!ihost) {
-      sock_err("Unknown host: %s\n", host);
-      return -1;
-    }
-    memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
-  }
-  
-  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-    int err = errno;
-
-    close(sock);
-    errno = err;
-    sock_err("Unable to connect %s[%s]: %s\n", host, service, strerror(errno));
-    return -1;
-  }
-
-  return sock;
-}
-
-int sock_serv(const char *service, const char *transport, int queue_length) {
-  struct sockaddr_in  sin;
-  int                 sock;
-  int                 on = 1;
-
-  sock = sock_create(service, transport, &sin);
-  
-  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
-  
-  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
-    sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
-  
-  if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
-    sock_err("Passive mode impossible on %s: %s\n", service, strerror(errno));
-  
-  return sock;
-}
-
 /*
  * Check for socket validity.
  */
-int sock_check_opened(int socket) {
+static int sock_check_opened(int socket) {
   fd_set   readfds, writefds, exceptfds;
   int      retval;
   struct   timeval timeout;
@@ -520,46 +512,6 @@ int sock_check_opened(int socket) {
   return 0;
 }
 
-/*
- * Read from socket.
- */
-int sock_read(int socket, char *buf, int len) {
-  char    *pbuf;
-  int      r, rr;
-  void    *nl;
-  
-  if((socket < 0) || (buf == NULL))
-    return -1;
-
-  if(!sock_check_opened(socket))
-    return -1;
-  
-  if (--len < 1)
-    return(-1);
-  
-  pbuf = buf;
-  
-  do {
-    
-    if((r = recv(socket, pbuf, len, MSG_PEEK)) <= 0)
-      return -1;
-
-    if((nl = memchr(pbuf, '\n', r)) != NULL)
-      r = ((char *) nl) - pbuf + 1;
-    
-    if((rr = read(socket, pbuf, r)) < 0)
-      return -1;
-    
-    pbuf += rr;
-    len -= rr;
-
-  } while((nl == NULL) && len);
-  
-  *pbuf = '\0';
-  
-  return (pbuf - buf);
-}
-
 /*
  * Write to socket.
  */
@@ -639,6 +591,34 @@ static char *_atoa(char *str) {
 }
 
 #ifdef NETWORK_CLIENT
+static int sock_client(const char *host, const char *service, const char *transport) {
+  struct sockaddr_in   sin;
+  struct hostent      *ihost;
+  int                  sock;
+  
+  sock = sock_create(service, transport, &sin);
+
+  if ((sin.sin_addr.s_addr = inet_addr(host)) == -1) {
+    ihost = gethostbyname(host);
+    
+    if(!ihost) {
+      sock_err("Unknown host: %s\n", host);
+      return -1;
+    }
+    memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
+  }
+  
+  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
+    int err = errno;
+
+    close(sock);
+    errno = err;
+    sock_err("Unable to connect %s[%s]: %s\n", host, service, strerror(errno));
+    return -1;
+  }
+
+  return sock;
+}
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 91)
 #define write_to_console_unlocked(session, msg, args...)  __sock_write(session->console, 1, msg, ##args)
@@ -760,7 +740,7 @@ static void client_help(session_t *session, session_commands_t *command, const c
   maxlen++; 
   i = 0;  
   
-  sprintf(buf, "%s", "Available commands are:\n       ");
+  snprintf(buf, sizeof(buf), "%s", "Available commands are:\n       ");
   curpos += 7;
   
   while(session_commands[i]->command != NULL) {
@@ -824,7 +804,7 @@ static void client_open(session_t *session, session_commands_t *command, const c
   }
   
   if(cmd) {
-    sprintf(buf, "%s", cmd);
+    snprintf(buf, sizeof(buf), "%s", cmd);
     pbuf = buf;
     if((p = strchr(pbuf, ' ')) != NULL) {
       host = _atoa(p);
@@ -1055,7 +1035,7 @@ static void client_handle_command(session_t *session, const char *command) {
   
   /* Get only first arg of command */
   memset(&cmd, 0, sizeof(cmd));
-  sprintf(cmd, "%s", command);
+  snprintf(cmd, sizeof(cmd), "%s", command);
   if((p = strchr(cmd, ' ')) != NULL)
     *p = '\0';
   
@@ -1134,7 +1114,7 @@ static void session_single_shot(session_t *session, int num_commands, char *comm
     if(strlen(buf))
       sprintf(buf, "%s %s", buf, commands[i]);
     else
-      sprintf(buf, "%s", commands[i]);
+      snprintf(buf, sizeof(buf), "%s", commands[i]);
   }
 
   client_handle_command(session, buf);
@@ -1144,7 +1124,7 @@ static void session_single_shot(session_t *session, int num_commands, char *comm
 
 static void show_version(void) {
   printf("This is %s - xine's remote control v%s.\n"
-	 "(c) 2000-2003 by G. Bartsch and the xine project team.\n", PROGNAME, PROGVERSION);
+	 "(c) 2000-2003 The xine Team.\n", PROGNAME, PROGVERSION);
 }
 
 static void show_usage(void) {
@@ -1345,6 +1325,64 @@ int main(int argc, char **argv) {
 
 #else
 
+static int sock_serv(const char *service, const char *transport, int queue_length) {
+  struct sockaddr_in  sin;
+  int                 sock;
+  int                 on = 1;
+
+  sock = sock_create(service, transport, &sin);
+  
+  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
+  
+  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
+    sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
+  
+  if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
+    sock_err("Passive mode impossible on %s: %s\n", service, strerror(errno));
+  
+  return sock;
+}
+
+/*
+ * Read from socket.
+ */
+static int sock_read(int socket, char *buf, int len) {
+  char    *pbuf;
+  int      r, rr;
+  void    *nl;
+  
+  if((socket < 0) || (buf == NULL))
+    return -1;
+
+  if(!sock_check_opened(socket))
+    return -1;
+  
+  if (--len < 1)
+    return(-1);
+  
+  pbuf = buf;
+  
+  do {
+    
+    if((r = recv(socket, pbuf, len, MSG_PEEK)) <= 0)
+      return -1;
+
+    if((nl = memchr(pbuf, '\n', r)) != NULL)
+      r = ((char *) nl) - pbuf + 1;
+    
+    if((rr = read(socket, pbuf, r)) < 0)
+      return -1;
+    
+    pbuf += rr;
+    len -= rr;
+
+  } while((nl == NULL) && len);
+  
+  *pbuf = '\0';
+  
+  return (pbuf - buf);
+}
+
 /*
  * Password related
  */
@@ -1392,7 +1430,7 @@ static int _passwdfile_is_entry_valid(char *entry, char *name, char *passwd) {
   char buf[_BUFSIZ];
   char *n, *p;
   
-  sprintf(buf, "%s", entry);
+  snprintf(buf, sizeof(buf), "%s", entry);
   n = buf;
   if((p = strrchr(buf, ':')) != NULL) {
     if(strlen(p) > 1) {
@@ -1605,7 +1643,7 @@ static const char *get_arg(client_info_t *client_info, int num) {
 /*
  * return 1 if *arg match with argument <pos>
  */
-int is_arg_contain(client_info_t *client_info, int pos, const char *arg) {
+static int is_arg_contain(client_info_t *client_info, int pos, const char *arg) {
   
   if(client_info && pos && ((arg != NULL) && (strlen(arg))) && (client_info->command.num_args >= pos)) {
     if(!strncmp(client_info->command.args[pos - 1], arg, strlen(client_info->command.args[pos - 1])))
@@ -1622,10 +1660,7 @@ static void set_command_line(client_info_t *client_info, char *line) {
 
   if(client_info && line && strlen(line)) {
 
-    if(client_info->command.line)
-      client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
-    else
-      client_info->command.line = (char *) xine_xmalloc(strlen(line) + 1);
+    client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
     
     memset(client_info->command.line, 0, sizeof(client_info->command.line));
     sprintf(client_info->command.line, "%s", line);
@@ -1670,7 +1705,7 @@ static void do_commands(commands_t *cmd, client_info_t *client_info) {
   char buf[_BUFSIZ];
 
   memset(&buf, 0, sizeof(buf));
-  sprintf(buf, COMMANDS_PREFIX);
+  snprintf(buf, sizeof(buf), "%s", COMMANDS_PREFIX);
   
   while(commands[i].command != NULL) {
     if(commands[i].public) {
@@ -1703,7 +1738,7 @@ static void do_help(commands_t *cmd, client_info_t *client_info) {
     maxlen++; 
     i = 0;  
 
-    sprintf(buf, "Available commands are:\n       ");
+    snprintf(buf, sizeof(buf), "%s", "Available commands are:\n       ");
     curpos += 7;
     
     while(commands[i].command != NULL) {
@@ -1764,7 +1799,7 @@ static void do_auth(commands_t *cmd, client_info_t *client_info) {
       char *name;
       char *passwd;
       
-      sprintf(buf, "%s", get_arg(client_info, 1));
+      snprintf(buf, sizeof(buf), "%s", get_arg(client_info, 1));
       name = buf;
       if((passwd = strrchr(buf, ':')) != NULL) {
 	if(strlen(passwd) > 1) {
@@ -1808,10 +1843,10 @@ static void do_mrl(commands_t *cmd, client_info_t *client_info) {
 	xine_stop (gGui->stream);
 	gGui->playlist.cur--;
 	if ((gGui->playlist.cur >= 0) && (gGui->playlist.cur < gGui->playlist.num)) {
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	  (void) gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-					gGui->mmk.start, gGui->mmk.offset);
-	  
+					gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1);
+  
 	} 
 	else {
 	  gGui->playlist.cur = 0;
@@ -1838,7 +1873,7 @@ static void do_mrl(commands_t *cmd, client_info_t *client_info) {
 	  xine_stop(gGui->stream);
 	  gGui->ignore_next = 0;
 	}
-	gui_set_current_mrl(gGui->playlist.mmk[gGui->playlist.num - 1]);
+	gui_set_current_mmk(gGui->playlist.mmk[gGui->playlist.num - 1]);
 	if(!(xine_open(gGui->stream, gGui->mmk.mrl) 
 	     && xine_play (gGui->stream, 0, gGui->mmk.start))) {
 	  handle_xine_error(client_info);
@@ -1857,11 +1892,15 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
   nargs = is_args(client_info);
   if(nargs) {
     if(nargs == 1) {
+      int first = 0;
+
       if(is_arg_contain(client_info, 1, "show")) {
 	int i;
+
 	if(gGui->playlist.num) {
 	  for(i = 0; i < gGui->playlist.num; i++) {
-	    sock_write(client_info->socket, "%5d %s\n", i, gGui->playlist.mmk[i]->mrl);
+	    sock_write(client_info->socket, "%2s %5d %s\n", 
+		       (i == gGui->playlist.cur) ? "*>" : "", i, gGui->playlist.mmk[i]->mrl);
 	  }
 	}
 	else
@@ -1877,6 +1916,39 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	set_command_line(client_info, "mrl prev");
 	handle_client_command(client_info);
       }
+      else if((first = is_arg_contain(client_info, 1, "first")) || is_arg_contain(client_info, 1, "last")) {
+
+	if(gGui->playlist.num) {
+	  int entry = gGui->playlist.cur;
+	  
+	  if(entry)
+	    entry = 0;
+	  else
+	    entry = gGui->playlist.num - 1;
+	  
+	  if(entry != gGui->playlist.cur) {
+	    
+	    if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+	      gGui->ignore_next = 1;
+	      xine_stop (gGui->stream);
+	      gGui->ignore_next = 0;
+	    }
+
+	    gGui->playlist.cur = entry;
+	    gui_set_current_mmk(mediamark_get_current_mmk());
+	    gui_play(NULL, NULL);
+	  }
+	}
+      }
+      else if(is_arg_contain(client_info, 1, "stop")) {
+	if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
+	  gGui->playlist.control |= PLAYLIST_CONTROL_STOP;
+      }
+      else if(is_arg_contain(client_info, 1, "continue")) {
+	if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
+	  gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+      }
+
     }
     else if(nargs >= 2) {
       
@@ -1886,7 +1958,16 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	j = atoi(get_arg(client_info, 2));
 	
 	if((j >= 0) && (j <= gGui->playlist.num)) {
-	  gui_set_current_mrl(gGui->playlist.mmk[j]);
+	  gGui->playlist.cur = j;
+	  gui_set_current_mmk(mediamark_get_current_mmk());
+
+	  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
+	    gGui->ignore_next = 1;
+	    xine_stop(gGui->stream);
+	    gGui->ignore_next = 0;
+
+	    do_play(cmd, client_info);
+	  }
 	}
       }
       else if(is_arg_contain(client_info, 1, "delete")) {
@@ -1917,8 +1998,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	    for(i = j; i < gGui->playlist.num; i++)
 	      gGui->playlist.mmk[i] = gGui->playlist.mmk[i + 1];
 	    
-	    gGui->playlist.mmk = (mediamark_t **) 
-	      realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
+	    gGui->playlist.mmk = (mediamark_t **) realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
 	    
 	    gGui->playlist.mmk[gGui->playlist.num + 1] = NULL;
 	    
@@ -1930,7 +2010,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	  playlist_update_playlist();
 
 	if(gGui->playlist.num)
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	else {
 	  
 	  if(is_playback_widgets_enabled())
@@ -1939,7 +2019,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
 	    gui_stop(NULL, NULL);
 	  
-	  gui_set_current_mrl(NULL);
+	  gui_set_current_mmk(NULL);
 	}
       }
     }
@@ -1989,11 +2069,13 @@ static void do_fullscreen(commands_t *cmd, client_info_t *client_info) {
   gui_execute_action_id(action);
 }
 
+#ifdef HAVE_XINERAMA
 static void do_xinerama_fullscreen(commands_t *cmd, client_info_t *client_info) {
-  action_id_t action = ACTID_TOGGLE_XINERAMA_FULLSCR;
+  action_id_t action = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
 
   gui_execute_action_id(action);
 }
+#endif
 
 static void do_get(commands_t *cmd, client_info_t *client_info) {
   int nargs;
@@ -2005,57 +2087,34 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	char buf[64];
 	int  status;
 
-	sprintf(buf, "%s", "Current status: ");
+	snprintf(buf, sizeof(buf), "%s", "Current status: ");
 	status = xine_get_status(gGui->stream);
 	
-	switch(status) {
-	case XINE_STATUS_STOP:
-	  sprintf(buf, "%s%s", buf, "XINE_STATUS_STOP");
-	  break;
-	case XINE_STATUS_PLAY:
-	  sprintf(buf, "%s%s", buf, "XINE_STATUS_PLAY");
-	  break;
-	case XINE_STATUS_QUIT:
-	  sprintf(buf, "%s%s", buf, "XINE_STATUS_QUIT");
-	  break;
-	default:
+	if(status <= XINE_STATUS_QUIT)
+	  sprintf(buf, "%s%s", buf, status_struct[status].name);
+	else
 	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
-	  break;
-	}
+	
 	sprintf(buf, "%s%c", buf, '\n');
 	sock_write(client_info->socket, buf);
       }
       else if(is_arg_contain(client_info, 1, "speed")) {
 	char buf[64];
-	int  speed;
+	int  speed = -1, i;
 	
-	sprintf(buf, "%s", "Current speed: ");
+	snprintf(buf, sizeof(buf), "%s", "Current speed: ");
 	speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
-	
-	switch(speed) {
-	case XINE_SPEED_PAUSE:
-	  sprintf(buf, "%s%s", buf, "XINE_SPEED_PAUSE");
-	  break;
-	case XINE_SPEED_SLOW_4:
-	  sprintf(buf, "%s%s", buf, "XINE_SPEED_SLOW_4");
-	  break;
-	case XINE_SPEED_SLOW_2:
-	  sprintf(buf, "%s%s", buf, "XINE_SPEED_SLOW_2");
-	  break;
-	case XINE_SPEED_NORMAL:
-	  sprintf(buf, "%s%s", buf, "XINE_SPEED_NORMAL");
-	  break;
-	case XINE_SPEED_FAST_2:
-	  sprintf(buf, "%s%s", buf, "XINE_SPEED_FAST_2");
-	  break;
-	case XINE_SPEED_FAST_4:
-	  sprintf(buf, "%s%s", buf, "XINE_SPEED_FAST_4");
-	  break;
-	default:
-	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
-	  break;
-	}
 
+	for(i = 0; speeds_struct[i].name != NULL; i++) {
+	  if(speed == speeds_struct[i].speed)
+	    break;
+	}
+	
+	if(i < ((sizeof(speeds_struct) / sizeof(speeds_struct[0])) - 1))
+	  sprintf(buf, "%s%s", buf, speeds_struct[i].name);
+	else
+	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
+	
 	sprintf(buf, "%s%c", buf, '\n');
 	sock_write(client_info->socket, buf);
       }
@@ -2068,7 +2127,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_stream,
 			    &pos_time,
 			    &length_time);
-	sprintf(buf, "Current position: %d\n",pos_time);
+	snprintf(buf, sizeof(buf), "%s: %d\n", "Current position", pos_time);
 	sock_write(client_info->socket, buf);
       }
       else if(is_arg_contain(client_info, 1, "length")) {
@@ -2080,32 +2139,32 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_stream,
 			    &pos_time,
 			    &length_time);
-	sprintf(buf, "Current length: %d\n",length_time);
+	snprintf(buf, sizeof(buf), "%s: %d\n", "Current length", length_time);
 	sock_write(client_info->socket, buf);
       }
       else if(is_arg_contain(client_info, 1, "loop")) {
 	char buf[64];
 
-	sprintf(buf, "Current loop mode is: ");
+	snprintf(buf, sizeof(buf), "%s", "Current loop mode is: ");
 
 	switch(gGui->playlist.loop) {
 	case PLAYLIST_LOOP_NO_LOOP:
-	  sprintf(buf, "%s'No Loop'", buf);
+	  sprintf(buf, "%s%s", buf, "'No Loop'");
 	  break;
 	case PLAYLIST_LOOP_LOOP:
-	  sprintf(buf, "%s'Loop'", buf);
+	  sprintf(buf, "%s%s", buf, "'Loop'");
 	  break;
 	case PLAYLIST_LOOP_REPEAT:
-	  sprintf(buf, "%s'Repeat'", buf);
+	  sprintf(buf, "%s%s", buf, "'Repeat'");
 	  break;
 	case PLAYLIST_LOOP_SHUFFLE:
-	  sprintf(buf, "%s'Shuffle'", buf);
+	  sprintf(buf, "%s%s", buf, "'Shuffle'");
 	  break;
 	case PLAYLIST_LOOP_SHUF_PLUS:	
-	  sprintf(buf, "%s'Shuffle forever'", buf);
+	  sprintf(buf, "%s%s", buf, "'Shuffle forever'");
 	  break;
 	default:
-	  sprintf(buf, "%s'!!Unknown!!'", buf);
+	  sprintf(buf, "%s%s", buf, "'!!Unknown!!'");
 	  break;
 	}
 
@@ -2120,7 +2179,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 		     (xine_get_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL)));
 	}
 	else if(is_arg_contain(client_info, 2, "lang")) {
-	  char buf[20];
+	  char buf[XINE_LANG_MAX];
 
 	  xine_get_audio_lang(gGui->stream,
 	                      xine_get_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL),
@@ -2148,13 +2207,27 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 		     (xine_get_param(gGui->stream, XINE_PARAM_SPU_CHANNEL)));
 	}
 	else if(is_arg_contain(client_info, 2, "lang")) {
-	  char buf[20];
+	  char buf[XINE_LANG_MAX];
 
 	  xine_get_spu_lang (gGui->stream,
 	                     xine_get_param(gGui->stream, XINE_PARAM_SPU_CHANNEL),
 			     &buf[0]);
 	  sock_write(client_info->socket, "Current spu language: %s\n", buf);
 	}
+	else if(is_arg_contain(client_info, 2, "offset")) {
+	  int offset;
+
+	  offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET);
+	  sock_write(client_info->socket, "Current spu offset: %d\n", offset);
+	}
+      }
+      else if(is_arg_contain(client_info, 1, "av")) {
+	if(is_arg_contain(client_info, 2, "offset")) {
+	  int offset;
+	  
+	  offset = xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET);
+	  sock_write(client_info->socket, "Current A/V offset: %d\n", offset);
+	}
       }
     }
   }
@@ -2168,7 +2241,7 @@ static void do_set(commands_t *cmd, client_info_t *client_info) {
     if(nargs == 2) {
       if(is_arg_contain(client_info, 1, "speed")) {
 	int speed;
-	
+
 	if((is_arg_contain(client_info, 2, "XINE_SPEED_PAUSE")) || 
 	   (is_arg_contain(client_info, 2, "|")))
 	  speed = XINE_SPEED_PAUSE;
@@ -2246,6 +2319,14 @@ static void do_set(commands_t *cmd, client_info_t *client_info) {
 	if(is_arg_contain(client_info, 2, "channel")) {
 	  xine_set_param(gGui->stream, XINE_PARAM_SPU_CHANNEL, (atoi(get_arg(client_info, 3))));
 	}
+	else if(is_arg_contain(client_info, 2, "offset")) {
+	  xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, (atoi(get_arg(client_info, 3))));
+	}
+      }
+      else if(is_arg_contain(client_info, 1, "av")) {
+	if(is_arg_contain(client_info, 2, "offset")) {
+	  xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, (atoi(get_arg(client_info, 3))));
+	}
       }
     }
   }
@@ -2289,6 +2370,14 @@ static void do_gui(commands_t *cmd, client_info_t *client_info) {
 	gui_setup_show(NULL, NULL);
 	flushing++;
       }
+      else if(is_arg_contain(client_info, 1, "vpost")) {
+	gui_vpp_show(NULL, NULL);
+	flushing++;
+      }
+      else if(is_arg_contain(client_info, 1, "help")) {
+	gui_help_show(NULL, NULL);
+	flushing++;
+      }
       else if(is_arg_contain(client_info, 1, "log")) {
 	gui_viewlog_show(NULL, NULL);
 	flushing++;
@@ -2314,90 +2403,52 @@ static void do_event(commands_t *cmd, client_info_t *client_info) {
     xine_event.type = 0;
 
     if(nargs == 1) {
-      if(is_arg_contain(client_info, 1, "menu1")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU1;
+      const char *arg = get_arg(client_info, 1);
+      struct {
+	char     *name;
+	int       type;
+      } events_struct[] = {
+	{ "menu1"   , XINE_EVENT_INPUT_MENU1    },
+	{ "menu2"   , XINE_EVENT_INPUT_MENU2    },
+	{ "menu3"   , XINE_EVENT_INPUT_MENU3    },
+	{ "menu4"   , XINE_EVENT_INPUT_MENU4    },
+	{ "menu5"   , XINE_EVENT_INPUT_MENU5    },
+	{ "menu6"   , XINE_EVENT_INPUT_MENU6    },
+	{ "menu7"   , XINE_EVENT_INPUT_MENU7    },
+	{ "up"      , XINE_EVENT_INPUT_UP       },
+	{ "down"    , XINE_EVENT_INPUT_DOWN     },
+	{ "left"    , XINE_EVENT_INPUT_LEFT     },
+	{ "right"   , XINE_EVENT_INPUT_RIGHT    },
+	{ "select"  , XINE_EVENT_INPUT_SELECT   },
+	{ "next"    , XINE_EVENT_INPUT_NEXT     },
+	{ "previous", XINE_EVENT_INPUT_PREVIOUS },
+	{ NULL      , -1                        }
+      };
+      int i = -1;
+
+      if(strlen(arg)) {
+	i = 0;
+	while(events_struct[i].name && strncmp(arg, events_struct[i].name, strlen(arg))) i++;
       }
-      else if(is_arg_contain(client_info, 1, "menu2")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU2;
-      }
-      else if(is_arg_contain(client_info, 1, "menu3")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU3;
-      }
-      else if(is_arg_contain(client_info, 1, "menu4")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU4;
-      }
-      else if(is_arg_contain(client_info, 1, "menu5")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU5;
-      }
-      else if(is_arg_contain(client_info, 1, "menu6")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU6;
-      }
-      else if(is_arg_contain(client_info, 1, "menu7")) {
-	xine_event.type = XINE_EVENT_INPUT_MENU7;
-      }
-      else if(is_arg_contain(client_info, 1, "up")) {
-	xine_event.type = XINE_EVENT_INPUT_UP;
-      }
-      else if(is_arg_contain(client_info, 1, "down")) {
-	xine_event.type = XINE_EVENT_INPUT_DOWN;
-      }
-      else if(is_arg_contain(client_info, 1, "left")) {
-	xine_event.type = XINE_EVENT_INPUT_LEFT;
-      }
-      else if(is_arg_contain(client_info, 1, "right")) {
-	xine_event.type = XINE_EVENT_INPUT_RIGHT;
-      }
-      else if(is_arg_contain(client_info, 1, "next")) {
-	xine_event.type = XINE_EVENT_INPUT_NEXT;
-      }
-      else if(is_arg_contain(client_info, 1, "previous")) {
-	xine_event.type = XINE_EVENT_INPUT_PREVIOUS;
-      }
-      else if(is_arg_contain(client_info, 1, "select")) {
-	xine_event.type = XINE_EVENT_INPUT_SELECT;
+
+      if((i >= 0) && (i < ((sizeof(events_struct) / sizeof(events_struct[0])) - 1))) {
+	xine_event.type = events_struct[i].type;
       }
       else {
-	char *arg = (char *)get_arg(client_info, 1);
 	int   value;
+	int   input_numbers[] = {
+	  XINE_EVENT_INPUT_NUMBER_0, XINE_EVENT_INPUT_NUMBER_1, XINE_EVENT_INPUT_NUMBER_2,
+	  XINE_EVENT_INPUT_NUMBER_3, XINE_EVENT_INPUT_NUMBER_4, XINE_EVENT_INPUT_NUMBER_5,
+	  XINE_EVENT_INPUT_NUMBER_6, XINE_EVENT_INPUT_NUMBER_7, XINE_EVENT_INPUT_NUMBER_8,
+	  XINE_EVENT_INPUT_NUMBER_9, XINE_EVENT_INPUT_NUMBER_10_ADD
+	};
+	  
 
 	if(((sscanf(arg, "%d", &value)) == 1) ||
 	   ((sscanf(arg, "+%d", &value)) == 1)) {
 	  
-	  switch(value) {
-	  case 0:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_0;
-	    break;
-	  case 1:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_1;
-	    break;
-	  case 2:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_2;
-	    break;
-	  case 3:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_3;
-	    break;
-	  case 4:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_4;
-	    break;
-	  case 5:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_5;
-	    break;
-	  case 6:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_6;
-	    break;
-	  case 7:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_7;
-	    break;
-	  case 8:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_8;
-	    break;
-	  case 9:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_9;
-	    break;
-	  case 10:
-	    xine_event.type = XINE_EVENT_INPUT_NUMBER_10_ADD;
-	    break;
-	  }
+	  if((value >= 0) && (value < (sizeof(input_numbers) / sizeof(input_numbers[0]))))
+	    xine_event.type = input_numbers[value];
 	}
       }
     }
@@ -2517,10 +2568,10 @@ static void say_hello(client_info_t *client_info) {
   struct hostent *hp = NULL;
   
   if(!gethostname(myfqdn, 255) && (hp = gethostbyname(myfqdn)) != NULL) {
-    sprintf(buf, "%s %s %s remote server. Nice to meet you.\n", hp->h_name, PACKAGE, VERSION);
+    snprintf(buf, sizeof(buf), "%s %s %s %s\n", hp->h_name, PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
   else {
-    sprintf(buf, "%s %s remote server. Nice to meet you.\n", PACKAGE, VERSION);
+    snprintf(buf, sizeof(buf), "%s %s %s\n", PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
   sock_write(client_info->socket, buf);
   
@@ -2582,13 +2633,9 @@ static void parse_destock_remain(client_info_t *client_info) {
 	p++;
       
       if(p)
-	sprintf(remaining, "%s", (_atoa(p)));
+	snprintf(remaining, sizeof(remaining), "%s", (_atoa(p)));
       
-      if(client_info->command.line)
-	client_info->command.line = (char *) realloc(client_info->command.line, 
-						     sizeof(char *) * (strlen(c) + 1));
-      else
-	client_info->command.line = (char *) xine_xmalloc(sizeof(char *) * (strlen(c) + 1));
+      client_info->command.line = (char *) realloc(client_info->command.line, sizeof(char *) * (strlen(c) + 1));
       
       sprintf(client_info->command.line, "%s", c);
       
@@ -2618,12 +2665,8 @@ static void parse_destock_remain(client_info_t *client_info) {
       
     }
     else { /* no ';' in remain, copy AS IS remain to line */
-      if(client_info->command.line)
-	client_info->command.line = (char *) realloc(client_info->command.line, 
-						     sizeof(char *) * (strlen(client_info->command.remain) + 1));
-      else
-	client_info->command.line = (char *) xine_xmalloc(sizeof(char *) * 
-							  (strlen(client_info->command.remain) + 1));
+      client_info->command.line = (char *) realloc(client_info->command.line, 
+						   sizeof(char *) * (strlen(client_info->command.remain) + 1));
       
       sprintf(client_info->command.line, "%s", client_info->command.remain);
       
@@ -2674,11 +2717,8 @@ static void parse_command(client_info_t *client_info) {
     cmd = NULL;
 
   if(cmd) {
-    if(client_info->command.command)
-      client_info->command.command = (char *) realloc(client_info->command.command, 
-						      strlen(client_info->command.line) - strlen(cmd) + 1);
-    else
-      client_info->command.command = (char *) xine_xmalloc(strlen(client_info->command.line) - strlen(cmd) + 1);
+    client_info->command.command = (char *) realloc(client_info->command.command, 
+						    strlen(client_info->command.line) - strlen(cmd) + 1);
 
     memset(client_info->command.command, 0, sizeof(client_info->command.command));
     snprintf(client_info->command.command, 
@@ -2747,10 +2787,7 @@ static void parse_command(client_info_t *client_info) {
     }
   }
   else {
-    if(client_info->command.command)
-      client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
-    else
-      client_info->command.command = (char *) xine_xmalloc(strlen(client_info->command.line) + 1);
+    client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
     
     memset(client_info->command.command, 0, sizeof(client_info->command.command));
     sprintf(client_info->command.command, "%s", client_info->command.line);
@@ -2864,8 +2901,7 @@ static void *client_thread(void *data) {
   for(i = 0; i < 256; i++)
     SAFE_FREE(client_info->command.args[i]);
   
-  SAFE_FREE(client_info);
-
+  free(client_info);
   client_info = NULL;
 
   pthread_exit(NULL);
@@ -2878,14 +2914,22 @@ static void *server_thread(void *data) {
   char            *service;
   struct servent  *serv_ent;
   int              msock;
+  
+  if(gGui->network_port) {
+    int len = (int) log10(gGui->network_port) + 1;
 
-  /*  Search in /etc/services if a xinectl entry exist */
-  if((serv_ent = getservbyname("xinectl", "tcp")) != NULL) {
-    service = (char *) xine_xmalloc(ntohs(serv_ent->s_port));
-    sprintf(service, "%u", ntohs(serv_ent->s_port));
-  }
-  else
-    service = strdup(DEFAULT_XINECTL_PORT);
+    service = (char *) xine_xmalloc(len + 1);
+    sprintf(service, "%u", gGui->network_port);
+  } 
+  else {
+    /*  Search in /etc/services if a xinectl entry exist */
+    if((serv_ent = getservbyname("xinectl", "tcp")) != NULL) {
+      service = (char *) xine_xmalloc(ntohs(serv_ent->s_port));
+      sprintf(service, "%u", ntohs(serv_ent->s_port));
+    }
+    else
+      service = strdup(DEFAULT_XINECTL_PORT);
+  }  
   
   /* Load passwd file */
   /* password file syntax is:
@@ -2917,7 +2961,7 @@ static void *server_thread(void *data) {
     char         passwdfilename[(strlen((get_homedir())) + strlen(passwdfile)) + 2];
     struct stat  st;
     
-    sprintf(passwdfilename, "%s/%s", (get_homedir()), passwdfile);
+    snprintf(passwdfilename, sizeof(passwdfilename), "%s/%s", (get_homedir()), passwdfile);
     
     if(stat(passwdfilename, &st) < 0) {
       fprintf(stderr, "ERROR: there is no password file for network access.!\n");
@@ -2946,10 +2990,13 @@ static void *server_thread(void *data) {
     
     if(client_info->socket < 0) {
       
+      free(client_info);
+      
       if(errno == EINTR)
 	continue;
       
       sock_err("accept: %s\n", strerror(errno));
+      continue;
     }
     
     close(msock);
@@ -2979,45 +3026,48 @@ static void *server_thread(void *data) {
  *
  */
 void start_remote_server(void) {
+  pthread_t thread;
 
-  if(gGui->network) {
-    pthread_create(&thread_server, NULL, server_thread, NULL);
-  }
+  if(gGui->network)
+    pthread_create(&thread, NULL, server_thread, NULL);
   
 }
 
-static const char *get_homedir(void) {
-  struct passwd *pw = NULL;
-  char *homedir = NULL;
+const char *get_homedir(void) {
 #ifdef HAVE_GETPWUID_R
-  int ret;
   struct passwd pwd;
-  char *buffer = NULL;
-  int bufsize = 128;
-  
-  buffer = (char *) malloc(bufsize);
-  memset(buffer, 0, sizeof(buffer));
-  
-  if((ret = getpwuid_r(getuid(), &pwd, buffer, bufsize, &pw)) < 0) {
+#endif
+  struct passwd *pw = NULL;
+  static char homedir[BUFSIZ] = {0,};
+
+  if(homedir[0])
+    return homedir;
+
+#ifdef HAVE_GETPWUID_R
+  if(getpwuid_r(getuid(), &pwd, homedir, sizeof(homedir), &pw) != 0 || pw == NULL) {
 #else
   if((pw = getpwuid(getuid())) == NULL) {
 #endif
-    if((homedir = getenv("HOME")) == NULL) {
-      fprintf(stderr, "Unable to get home directory, set it to /tmp.\n");
-      homedir = strdup("/tmp");
+    char *tmp = getenv("HOME");
+    if(tmp) {
+      strncpy(homedir, tmp, sizeof(homedir));
+      homedir[sizeof(homedir) - 1] = '\0';
     }
+  } else {
+    char *s = strdup(pw->pw_dir);
+    strncpy(homedir, s, sizeof(homedir));
+    homedir[sizeof(homedir) - 1] = '\0';
+    free(s);
   }
-  else {
-    if(pw) 
-      homedir = strdup(pw->pw_dir);
+
+  if(!homedir[0]) {
+    printf("Unable to get home directory, set it to /tmp.\n");
+    strcpy(homedir, "/tmp");
   }
-  
-  
-#ifdef HAVE_GETPWUID_R
-  SAFE_FREE(buffer);
-#endif
-  
+
   return homedir;
 }
 
 #endif
+
+#endif  /* HAVE_READLINE */
