@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: labelbutton.c,v 1.59 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: labelbutton.c,v 1.60 2004/06/13 02:36:35 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,10 @@ static void notify_destroy(xitk_widget_t *w) {
 
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
+
+    if(!private_data->skin_element_name)
+      xitk_image_free_image(private_data->imlibdata, &(private_data->skin));
+
     XITK_FREE(private_data->label);
     XITK_FREE(private_data->shortcut_label);
     XITK_FREE(private_data->shortcut_font);
