@@ -112,6 +112,7 @@ static int init_test(void) {
   int                screen;
   int		     depth = 0;
   Visual	    *visual = NULL;
+  XColor             black, dummy;
   
   if(!XInitThreads ()) {
     printf (_("XInitThreads() failed.\n"));
@@ -139,7 +140,9 @@ static int init_test(void) {
     return 0;
   }
 
-  xitk_init(test->display, 1);
+  XAllocNamedColor(test->display, Imlib_get_colormap(test->imlibdata), "black", &black, &dummy);
+
+  xitk_init(test->display, black, 1);
 
   XUnlockDisplay (test->display);
 
@@ -178,7 +181,7 @@ static void test_end(xitk_widget_t *w, void *data) {
 /*
  *
  */
-void test_handle_event(XEvent *event, void *data) {
+static void test_handle_event(XEvent *event, void *data) {
   XKeyEvent      mykeyevent;
   KeySym         mykey;
   char           kbuf[256];
@@ -340,7 +343,7 @@ static void change_browser_entry(xitk_widget_t *w, void *data, char *currenttext
   if((j = xitk_browser_get_current_selected(test->browser)) >= 0) {
     free(test->entries[j]);
     test->entries[j] = strdup(currenttext);
-    xitk_browser_rebuild_browser(test->browser, j);
+    xitk_browser_rebuild_browser(test->browser, xitk_browser_get_current_start(test->browser));
   }
 }
 
@@ -451,6 +454,12 @@ static void create_doublebox(void) {
 /*
  *
  */
+static void checkbox_cb(xitk_widget_t *w, void *data, int state) {
+  if(state)
+    xitk_cursors_restore_window_cursor(test->display, xitk_window_get_window(test->xwin));
+  else
+    xitk_cursors_define_window_cursor(test->display, xitk_window_get_window(test->xwin), xitk_cursor_invisible);
+}
 static void create_checkbox(void) {
   int x = 250, y = 300;
   xitk_checkbox_widget_t cb;
@@ -458,7 +467,7 @@ static void create_checkbox(void) {
   XITK_WIDGET_INIT(&cb, test->imlibdata);
 
   cb.skin_element_name = NULL;
-  cb.callback          = NULL;
+  cb.callback          = checkbox_cb;
   cb.userdata          = NULL;
   xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list),
 		    (test->checkbox = 
@@ -811,7 +820,7 @@ static void create_browser(void) {
 
   xitk_window_change_background(test->imlibdata, 
 				test->xwin, bg, width, height);
-
+  XFreePixmap(test->display, bg);
 
   {
     int i;
@@ -819,7 +828,7 @@ static void create_browser(void) {
     
     for(i = 0; i < 25; i++) {
       memset(&buf, 0, sizeof(buf));
-      sprintf(buf, "Entry %d", i);
+      snprintf(buf, sizeof(buf), "Entry %d", i);
       test->entries[i] = strdup(buf);
     }
     
@@ -934,6 +943,10 @@ static void create_menu(void) {
   xitk_menu_show_menu(test->menu);
 }
 
+static void test_dndcb(char *file) {
+  printf("file: '%s'\n", file);
+}
+
 /*
  *
  */
@@ -958,7 +971,6 @@ int main(int argc, char **argv) {
   bindtextdomain("xitk", XITK_LOCALE);
   textdomain("xitk");
 
-
   /* Create window */
   test->xwin = xitk_window_create_dialog_window(test->imlibdata,
 						_("My Fucking Window"), 
@@ -1008,7 +1020,7 @@ int main(int argc, char **argv) {
 					   (xitk_window_get_window(test->xwin)),
 					   test_handle_event,
 					   NULL,
-					   NULL,
+					   test_dndcb,
 					   test->widget_list,
 					   NULL);
 
