@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: window.c,v 1.43 2004/04/13 19:55:50 f1rmb Exp $
+ * $Id: window.c,v 1.50 2004/12/16 21:36:02 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -41,6 +41,45 @@ static void _xitk_window_destroy_window(xitk_widget_t *, void *);
 
 #define TITLE_BAR_HEIGHT 20
 
+static void _xitk_window_set_focus(Display *display, Window window) {
+  int t = 0;
+
+  while((!xitk_is_window_visible(display, window)) && (++t < 3))
+    xitk_usec_sleep(5000);
+  
+  if(xitk_is_window_visible(display, window)) {
+    XLockDisplay(display);
+    XSetInputFocus(display, window, RevertToParent, CurrentTime);
+    XUnlockDisplay(display);
+  }
+}
+
+/*
+ *
+ */
+int xitk_is_window_iconified(Display *display, Window window) {
+  unsigned char *prop_return = NULL;
+  unsigned long  nitems_return;
+  unsigned long  bytes_after_return;
+  int            format_return;
+  Atom           type_return, atom;
+  int            retval = 0;
+  
+  XLockDisplay(display);
+  atom = XInternAtom(display, "WM_STATE", False);
+  XGetWindowProperty (display, window, atom, 0, 0x7fffffff, False,
+		      atom, &type_return, &format_return, &nitems_return, &bytes_after_return, &prop_return);
+  
+  if(prop_return) {
+    if (prop_return[0] == IconicState)
+      retval = 1;
+    XFree(prop_return);
+  }
+  XUnlockDisplay(display);
+ 
+  return retval;
+}
+
 /*
  * Is window is size match with given args
  */
@@ -200,7 +239,7 @@ void xitk_window_center_window(ImlibData *im, xitk_window_t *w) {
  */
 xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width, int height) {
   xitk_window_t         *xwin;
-  char                   title[] = {"Xitk Window"};
+  char                   title[] = {"xiTK Window"};
   XSizeHints             hint;
   XWMHints              *wm_hint;
   XSetWindowAttributes   attr;
@@ -213,11 +252,13 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
   if((im == NULL) || (width == 0 || height == 0))
     return NULL;
 
-  xwin             = (xitk_window_t *) xitk_xmalloc(sizeof(xitk_window_t));
-  xwin->background = NULL;
-  xwin->width      = width;
-  xwin->height     = height;
-  xwin->parent     = NULL;
+  xwin                  = (xitk_window_t *) xitk_xmalloc(sizeof(xitk_window_t));
+  xwin->win_parent      = None;
+  xwin->background      = NULL;
+  xwin->background_mask = NULL;
+  xwin->width           = width;
+  xwin->height          = height;
+  xwin->parent          = NULL;
   
   memset(&hint, 0, sizeof(hint));
   hint.x               = x;
@@ -476,6 +517,20 @@ Pixmap xitk_window_get_background(xitk_window_t *w) {
   return w->background->pixmap;
 }
 
+/*
+ * Return window background pixmap.
+ */
+Pixmap xitk_window_get_background_mask(xitk_window_t *w) {
+
+  if(w == NULL)
+    return None;
+
+  if(w->background_mask == NULL)
+    return None;
+  
+  return w->background->pixmap;
+}
+
 /*
  * Apply (draw) window background.
  */
@@ -485,7 +540,14 @@ void xitk_window_apply_background(ImlibData *im, xitk_window_t *w) {
     return;
 
   XLOCK(im->x.disp);
+  
   XSetWindowBackgroundPixmap(im->x.disp, w->window, w->background->pixmap);
+  
+  if(w->background_mask)
+    XShapeCombineMask(im->x.disp, w->window, ShapeBounding, 0, 0, w->background_mask->pixmap, ShapeSet);
+  else
+    XShapeCombineMask(im->x.disp, w->window, ShapeBounding, 0, 0, 0, ShapeSet);
+
   XClearWindow(im->x.disp, w->window);
   XUNLOCK(im->x.disp);
 }
@@ -503,7 +565,11 @@ int xitk_window_change_background(ImlibData *im,
     return 0;
 
   xitk_image_destroy_xitk_pixmap(w->background);
-  w->background = xitk_image_create_xitk_pixmap(im, width, height);
+  if(w->background_mask)
+    xitk_image_destroy_xitk_pixmap(w->background_mask);
+  
+  w->background      = xitk_image_create_xitk_pixmap(im, width, height);
+  w->background_mask = NULL;
   
   XLOCK(im->x.disp);
   if(XGetGeometry(im->x.disp, w->window, &rootwin, 
@@ -513,7 +579,7 @@ int xitk_window_change_background(ImlibData *im,
   }
   else
     return 0;
-
+  
   XCopyArea(im->x.disp, bg, w->background->pixmap, w->background->gc, 0, 0, width, height, 0, 0);
   XUNLOCK(im->x.disp);
 
@@ -522,6 +588,46 @@ int xitk_window_change_background(ImlibData *im,
   return 1;
 }
 
+/*
+ * Change window background with img, then draw it.
+ */
+int xitk_window_change_background_with_image(ImlibData *im, xitk_window_t *w, xitk_image_t *img, int width, int height) {
+  Window        rootwin;
+  int           xwin, ywin;
+  unsigned int  wwin, hwin, bwin, dwin;
+
+  if((im == NULL) || (w == NULL) || (img == NULL) || (width == 0 || height == 0))
+    return 0;
+  
+  xitk_image_destroy_xitk_pixmap(w->background);
+  if(w->background_mask)
+    xitk_image_destroy_xitk_pixmap(w->background_mask);
+
+  w->background      = xitk_image_create_xitk_pixmap(im, width, height);
+  w->background_mask = NULL;
+  
+  if(img->mask)
+    w->background_mask = xitk_image_create_xitk_mask_pixmap(im, width, height);
+  
+  XLOCK(im->x.disp);
+  if(XGetGeometry(im->x.disp, w->window, &rootwin, 
+		  &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
+    
+    XResizeWindow (im->x.disp, w->window, wwin, hwin);
+  }
+  else
+    return 0;
+  
+  XCopyArea(im->x.disp, img->image->pixmap, w->background->pixmap, w->background->gc, 0, 0, width, height, 0, 0);
+  if(w->background_mask)
+    XCopyArea(im->x.disp, img->mask->pixmap, w->background_mask->pixmap, w->background_mask->gc, 0, 0, width, height, 0, 0);
+  
+  XUNLOCK(im->x.disp);
+  xitk_window_apply_background(im, w);
+  
+  return 1;
+}
+
 /*
  * Local XEvent handling.
  */
@@ -616,6 +722,10 @@ void xitk_window_destroy_window(ImlibData *im, xitk_window_t *w) {
 
   XLOCK(im->x.disp);
   XDestroyWindow(im->x.disp, w->window);
+
+  if((w->win_parent != None) && xitk_is_window_visible(im->x.disp, w->win_parent))
+    XSetInputFocus(im->x.disp, w->win_parent, RevertToParent, CurrentTime);
+
   XUNLOCK(im->x.disp);
 
   XITK_FREE(w);
@@ -705,7 +815,7 @@ xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *ti
 							 int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   int                         windoww = window_width, windowh;
-  xitk_image_t               *i;
+  xitk_image_t               *image;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
     return NULL;
@@ -738,11 +848,11 @@ xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *ti
 	return NULL;
     }
     
-    i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
+    image = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
     XITK_FREE(buf);
   }
   
-  windowh = (i->height) + (TITLE_BAR_HEIGHT + 40);
+  windowh = (image->height) + (TITLE_BAR_HEIGHT + 40);
 
   wd->imlibdata = im;
   wd->type = DIALOG_TYPE_BUTTONLESS;
@@ -769,8 +879,8 @@ xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *ti
     gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
     XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
-	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, image->image->pixmap, bg->pixmap,
+	      image->image->gc, 0, 0, image->width, image->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
     xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
@@ -781,7 +891,7 @@ xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *ti
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
-    xitk_image_free_image(im, &i);
+    xitk_image_free_image(im, &image);
 
   }
 
@@ -797,6 +907,8 @@ xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *ti
 					NULL,
 					(void *)wd);
   
+  _xitk_window_set_focus(im->x.disp, (xitk_window_get_window(wd->xwin)));
+
   return wd->xwin;
 }
 /*
@@ -808,7 +920,7 @@ xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *tit
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
-  xitk_image_t               *i;
+  xitk_image_t               *image;
   int                         bwidth = 100, bx, by;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
@@ -842,11 +954,11 @@ xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *tit
 	return NULL;
     }
     
-    i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
+    image = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
     XITK_FREE(buf);
   }
   
-  windowh = (i->height + 50) + (TITLE_BAR_HEIGHT + 40);
+  windowh = (image->height + 50) + (TITLE_BAR_HEIGHT + 40);
 
   wd->imlibdata = im;
   wd->type = DIALOG_TYPE_OK;
@@ -903,8 +1015,8 @@ xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *tit
     gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
     XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
-	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, image->image->pixmap, bg->pixmap,
+	      image->image->gc, 0, 0, image->width, image->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
     xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
@@ -915,7 +1027,7 @@ xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *tit
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
-    xitk_image_free_image(im, &i);
+    xitk_image_free_image(im, &image);
 
   }
 
@@ -932,6 +1044,8 @@ xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *tit
 					(void *)wd);
 
   xitk_enable_and_show_widget(wd->wyes);
+  
+  _xitk_window_set_focus(im->x.disp, (xitk_window_get_window(wd->xwin)));
 
   return wd->xwin;
 }
@@ -976,20 +1090,31 @@ xitk_window_t *xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
   return xw;
 }
 
+static void _checkbox_label_click(xitk_widget_t *w, void *data) {
+  xitk_dialog_t *wd = (xitk_dialog_t *) data;
+  
+  xitk_checkbox_set_state(wd->checkbox, 
+    (!xitk_checkbox_get_state(wd->checkbox)));
+  xitk_checkbox_callback_exec(wd->checkbox);
+}
+
 /*
  * Create an interactive window, containing 'yes', 'no', 'cancel' buttons.
  */
-xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_checkbox_two_buttons_with_width(ImlibData *im, char *title,
 							 char *button1_label, char *button2_label,
 							 xitk_state_callback_t cb1, 
 							 xitk_state_callback_t cb2, 
+                                                         char *checkbox_label, int checkbox_state,
+                                                         xitk_state_callback_t cb3,
 							 void *userdata, 
 							 int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
-  xitk_image_t               *i;
-  int                         bwidth = 100, bx1, bx2, by;
+  xitk_image_t               *image;
+  int                         bwidth = 150, bx1, bx2, by;
+  int                         checkbox_height = 0;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
     return NULL;
@@ -1022,11 +1147,14 @@ xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *ti
 	return NULL;
     }
     
-    i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
+    image = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
     XITK_FREE(buf);
   }
+  
+  if( checkbox_label )
+    checkbox_height = 50;
 
-  windowh = (i->height + 50) + (TITLE_BAR_HEIGHT + 40);
+  windowh = (image->height + 50 + checkbox_height) + (TITLE_BAR_HEIGHT + 40);
 
   wd->imlibdata = im;
   wd->type = DIALOG_TYPE_YESNO;
@@ -1045,7 +1173,36 @@ xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *ti
   wd->widget_list->gc            = (XCreateGC(im->x.disp, (xitk_window_get_window(wd->xwin)),
 					      None, None));
   XUNLOCK(wd->imlibdata->x.disp);
-				
+		
+  /* Checkbox */
+  if( checkbox_label ) {
+    int x = 25, y = windowh - 50 - checkbox_height;
+    xitk_checkbox_widget_t cb;
+    xitk_label_widget_t lbl;
+  
+    XITK_WIDGET_INIT(&cb, im);
+    XITK_WIDGET_INIT(&lbl, im);
+  
+    cb.skin_element_name = NULL;
+    cb.callback          = cb3;
+    cb.userdata          = userdata;
+    xitk_list_append_content (XITK_WIDGET_LIST_LIST(wd->widget_list),
+                      (wd->checkbox = 
+                      xitk_noskin_checkbox_create(wd->widget_list, &cb, x, y+5, 10, 10)));
+  
+    xitk_checkbox_set_state(wd->checkbox, checkbox_state);
+  
+    lbl.window            = xitk_window_get_window(wd->xwin);
+    lbl.gc                = (XITK_WIDGET_LIST_GC(wd->widget_list));
+    lbl.skin_element_name = NULL;
+    lbl.label             = checkbox_label;
+    lbl.callback          = _checkbox_label_click;
+    lbl.userdata          = wd;
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(wd->widget_list)), 
+            (wd->checkbox_label = 
+              xitk_noskin_label_create(wd->widget_list, &lbl, x + 15, y, windoww - x - 40, 20, DEFAULT_FONT_12)));
+  }
+  
   /* Buttons */
   if((bwidth * 2) > windoww)
     bwidth = (windoww / 2) - 8;
@@ -1098,8 +1255,8 @@ xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *ti
     gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
     XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
-	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, image->image->pixmap, bg->pixmap,
+	      image->image->gc, 0, 0, image->width, image->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
     xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
@@ -1110,7 +1267,7 @@ xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *ti
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
-    xitk_image_free_image(im, &i);
+    xitk_image_free_image(im, &image);
 
   }
 
@@ -1127,6 +1284,12 @@ xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *ti
 					(void *)wd);
   xitk_enable_and_show_widget(wd->wyes);
   xitk_enable_and_show_widget(wd->wno);
+  if( checkbox_label ) {
+    xitk_enable_and_show_widget(wd->checkbox);
+    xitk_enable_and_show_widget(wd->checkbox_label);
+  }
+  
+  _xitk_window_set_focus(im->x.disp, (xitk_window_get_window(wd->xwin)));
 
   return wd->xwin;
 }
@@ -1192,7 +1355,7 @@ xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
-  xitk_image_t               *i;
+  xitk_image_t               *image;
   int                         bwidth = 100, bx1, bx2, bx3, by;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
@@ -1226,11 +1389,11 @@ xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *
 	return NULL;
     }
     
-    i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
+    image = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
     XITK_FREE(buf);
   }
 
-  windowh = (i->height + 50) + (TITLE_BAR_HEIGHT + 40);
+  windowh = (image->height + 50) + (TITLE_BAR_HEIGHT + 40);
 
   wd->imlibdata      = im;
   wd->type           = DIALOG_TYPE_YESNOCANCEL;
@@ -1320,8 +1483,8 @@ xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *
     gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
     XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
-	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, image->image->pixmap, bg->pixmap,
+	      image->image->gc, 0, 0, image->width, image->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
     xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
@@ -1331,7 +1494,7 @@ xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
-    xitk_image_free_image(im, &i);
+    xitk_image_free_image(im, &image);
 
   }
 
@@ -1350,6 +1513,8 @@ xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *
   xitk_enable_and_show_widget(wd->wno);
   xitk_enable_and_show_widget(wd->wcancel);
 
+  _xitk_window_set_focus(im->x.disp, (xitk_window_get_window(wd->xwin)));
+
   return wd->xwin;
 }
 
@@ -1395,3 +1560,8 @@ xitk_window_t *xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *ti
   XITK_FREE(buf);
   return xw;
 }
+
+void xitk_window_set_parent_window(xitk_window_t *xwin, Window parent) {
+  if(xwin)
+    xwin->win_parent = parent;
+}
